# file: autobyteus/autobyteus/agent/remote_agent.py
from autobyteus.agent.agent import Agent

class RemoteAgent(Agent):
    """
    A placeholder class for remote agents that will interact with external systems
    (e.g., via MCP or HTTP protocols). To be implemented in the future.
    """
    pass
